@@ -12,13 +12,18 @@
 #include "soup-client-input-stream.h"
 #include "soup.h"
 #include "soup-message-private.h"
+#include "soup-misc.h"
 
-struct _SoupClientInputStreamPrivate {
-	SoupMessage  *msg;
+struct _SoupClientInputStream {
+	SoupFilterInputStream parent_instance;
 };
 
+typedef struct {
+	SoupMessage  *msg;
+} SoupClientInputStreamPrivate;
+
 enum {
-	EOF,
+	SIGNAL_EOF,
 	LAST_SIGNAL
 };
 
@@ -34,23 +39,22 @@ static GPollableInputStreamInterface *soup_client_input_stream_parent_pollable_i
 static void soup_client_input_stream_pollable_init (GPollableInputStreamInterface *pollable_interface, gpointer interface_data);
 
 G_DEFINE_TYPE_WITH_CODE (SoupClientInputStream, soup_client_input_stream, SOUP_TYPE_FILTER_INPUT_STREAM,
+                         G_ADD_PRIVATE (SoupClientInputStream)
 			 G_IMPLEMENT_INTERFACE (G_TYPE_POLLABLE_INPUT_STREAM,
 						soup_client_input_stream_pollable_init))
 
 static void
 soup_client_input_stream_init (SoupClientInputStream *stream)
 {
-	stream->priv = G_TYPE_INSTANCE_GET_PRIVATE (stream,
-						    SOUP_TYPE_CLIENT_INPUT_STREAM,
-						    SoupClientInputStreamPrivate);
 }
 
 static void
 soup_client_input_stream_finalize (GObject *object)
 {
 	SoupClientInputStream *cistream = SOUP_CLIENT_INPUT_STREAM (object);
+        SoupClientInputStreamPrivate *priv = soup_client_input_stream_get_instance_private (cistream);
 
-	g_clear_object (&cistream->priv->msg);
+	g_clear_object (&priv->msg);
 
 	G_OBJECT_CLASS (soup_client_input_stream_parent_class)->finalize (object);
 }
@@ -60,10 +64,11 @@ soup_client_input_stream_set_property (GObject *object, guint prop_id,
 				       const GValue *value, GParamSpec *pspec)
 {
 	SoupClientInputStream *cistream = SOUP_CLIENT_INPUT_STREAM (object);
+        SoupClientInputStreamPrivate *priv = soup_client_input_stream_get_instance_private (cistream);
 
 	switch (prop_id) {
 	case PROP_MESSAGE:
-		cistream->priv->msg = g_value_dup_object (value);
+		priv->msg = g_value_dup_object (value);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -76,10 +81,11 @@ soup_client_input_stream_get_property (GObject *object, guint prop_id,
 				       GValue *value, GParamSpec *pspec)
 {
 	SoupClientInputStream *cistream = SOUP_CLIENT_INPUT_STREAM (object);
+        SoupClientInputStreamPrivate *priv = soup_client_input_stream_get_instance_private (cistream);
 
 	switch (prop_id) {
 	case PROP_MESSAGE:
-		g_value_set_object (value, cistream->priv->msg);
+		g_value_set_object (value, priv->msg);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -100,7 +106,7 @@ soup_client_input_stream_read_fn (GInputStream  *stream,
 		read_fn (stream, buffer, count, cancellable, error);
 
 	if (nread == 0)
-		g_signal_emit (stream, signals[EOF], 0);
+		g_signal_emit (stream, signals[SIGNAL_EOF], 0);
 
 	return nread;
 }
@@ -117,7 +123,7 @@ soup_client_input_stream_read_nonblocking (GPollableInputStream  *stream,
 		read_nonblocking (stream, buffer, count, error);
 
 	if (nread == 0)
-		g_signal_emit (stream, signals[EOF], 0);
+		g_signal_emit (stream, signals[SIGNAL_EOF], 0);
 
 	return nread;
 }
@@ -128,11 +134,12 @@ soup_client_input_stream_close_fn (GInputStream  *stream,
 				   GError       **error)
 {
 	SoupClientInputStream *cistream = SOUP_CLIENT_INPUT_STREAM (stream);
+        SoupClientInputStreamPrivate *priv = soup_client_input_stream_get_instance_private (cistream);
 	gboolean success;
 
-	success = soup_message_io_run_until_finish (cistream->priv->msg, TRUE,
+	success = soup_message_io_run_until_finish (priv->msg, TRUE,
 						    NULL, error);
-	soup_message_io_finished (cistream->priv->msg);
+	soup_message_io_finished (priv->msg);
 	return success;
 }
 
@@ -151,9 +158,10 @@ close_async_ready (SoupMessage *msg, gpointer user_data)
 {
 	GTask *task = user_data;
 	SoupClientInputStream *cistream = g_task_get_source_object (task);
+        SoupClientInputStreamPrivate *priv = soup_client_input_stream_get_instance_private (cistream);
 	GError *error = NULL;
 
-	if (!soup_message_io_run_until_finish (cistream->priv->msg, FALSE,
+	if (!soup_message_io_run_until_finish (priv->msg, FALSE,
 					       g_task_get_cancellable (task),
 					       &error) &&
 	    g_error_matches (error, G_IO_ERROR, G_IO_ERROR_WOULD_BLOCK)) {
@@ -161,7 +169,7 @@ close_async_ready (SoupMessage *msg, gpointer user_data)
 		return TRUE;
 	}
 
-	soup_message_io_finished (cistream->priv->msg);
+	soup_message_io_finished (priv->msg);
 
 	if (error) {
 		g_task_return_error (task, error);
@@ -173,8 +181,11 @@ close_async_ready (SoupMessage *msg, gpointer user_data)
 	 * waiting one extra cycle after run_until_finish() returns.
 	 * Ugh. FIXME later when it's easier to do.
 	 */
-	soup_add_idle (g_main_context_get_thread_default (),
-		       idle_finish_close, task);
+	GSource *source = g_idle_source_new ();
+	g_source_set_callback (source, idle_finish_close, task, NULL);
+	g_source_attach (source, g_main_context_get_thread_default ());
+	g_source_unref (source);
+
 	return FALSE;
 }
 
@@ -186,15 +197,17 @@ soup_client_input_stream_close_async (GInputStream        *stream,
 				      gpointer             user_data)
 {
 	SoupClientInputStream *cistream = SOUP_CLIENT_INPUT_STREAM (stream);
+        SoupClientInputStreamPrivate *priv = soup_client_input_stream_get_instance_private (cistream);
 	GTask *task;
 	GSource *source;
 
 	task = g_task_new (stream, cancellable, callback, user_data);
 	g_task_set_priority (task, priority);
 
-	if (close_async_ready (cistream->priv->msg, task) == G_SOURCE_CONTINUE) {
-		source = soup_message_io_get_source (cistream->priv->msg,
-						     cancellable, NULL, NULL);
+	if (close_async_ready (priv->msg, task) == G_SOURCE_CONTINUE) {
+		source = soup_message_io_data_get_source ((SoupMessageIOData *)soup_message_get_io_data (priv->msg),
+							  G_OBJECT (priv->msg),
+							  cancellable, NULL, NULL);
 
 		g_task_attach_source (task, source, (GSourceFunc) close_async_ready);
 		g_source_unref (source);
@@ -215,8 +228,6 @@ soup_client_input_stream_class_init (SoupClientInputStreamClass *stream_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (stream_class);
 	GInputStreamClass *input_stream_class = G_INPUT_STREAM_CLASS (stream_class);
 
-	g_type_class_add_private (stream_class, sizeof (SoupClientInputStreamPrivate));
-
 	object_class->finalize = soup_client_input_stream_finalize;
 	object_class->set_property = soup_client_input_stream_set_property;
 	object_class->get_property = soup_client_input_stream_get_property;
@@ -226,7 +237,7 @@ soup_client_input_stream_class_init (SoupClientInputStreamClass *stream_class)
 	input_stream_class->close_async = soup_client_input_stream_close_async;
 	input_stream_class->close_finish = soup_client_input_stream_close_finish;
 
-	signals[EOF] =
+	signals[SIGNAL_EOF] =
 		g_signal_new ("eof",
 			      G_OBJECT_CLASS_TYPE (object_class),
 			      G_SIGNAL_RUN_LAST,
@@ -241,7 +252,8 @@ soup_client_input_stream_class_init (SoupClientInputStreamClass *stream_class)
 				     "Message",
 				     "Message",
 				     SOUP_TYPE_MESSAGE,
-				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+				     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY |
+				     G_PARAM_STATIC_STRINGS));
 }
 
 static void
