@@ -12,9 +12,19 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "soup-misc.h"
 #include "soup-headers.h"
 #include "soup.h"
 
+/**
+ * SECTION:soup-headers
+ * @section_id: SoupHeaders
+ * @title: SoupHeaders
+ * @short_description: Functions to help working with HTTP Headers
+ *
+ * These are utility functions to help working with HTTP headers.
+ */
+
 /**
  * soup_headers_parse:
  * @str: the header string (including the Request-Line or Status-Line,
@@ -29,9 +39,8 @@
  * This is a low-level method; normally you would use
  * soup_headers_parse_request() or soup_headers_parse_response().
  *
- * Return value: success or failure
+ * Returns: success or failure
  *
- * Since: 2.26
  **/
 gboolean
 soup_headers_parse (const char *str, int len, SoupMessageHeaders *dest)
@@ -171,7 +180,7 @@ done:
  *
  * Beware that @req_headers may be modified even on failure.
  *
- * Return value: %SOUP_STATUS_OK if the headers could be parsed, or an
+ * Returns: %SOUP_STATUS_OK if the headers could be parsed, or an
  * HTTP error to be returned to the client if they could not be.
  **/
 guint
@@ -187,7 +196,7 @@ soup_headers_parse_request (const char          *str,
 	unsigned long major_version, minor_version;
 	char *p;
 
-	g_return_val_if_fail (str != NULL, SOUP_STATUS_MALFORMED);
+	g_return_val_if_fail (str != NULL, SOUP_STATUS_BAD_REQUEST);
 
 	/* RFC 2616 4.1 "servers SHOULD ignore any empty line(s)
 	 * received where a Request-Line is expected."
@@ -280,7 +289,7 @@ soup_headers_parse_request (const char          *str,
  * @status_code, and @reason_phrase. @status_line must be terminated by
  * either "\0" or "\r\n".
  *
- * Return value: %TRUE if @status_line was parsed successfully.
+ * Returns: %TRUE if @status_line was parsed successfully.
  **/
 gboolean
 soup_headers_parse_status_line (const char       *status_line,
@@ -358,7 +367,7 @@ soup_headers_parse_status_line (const char       *status_line,
  *
  * Beware that @headers may be modified even on failure.
  *
- * Return value: success or failure.
+ * Returns: success or failure.
  **/
 gboolean
 soup_headers_parse_response (const char          *str, 
@@ -482,7 +491,7 @@ parse_list (const char *header, char delim)
  * "#something", where "something" does not itself contain commas,
  * except as part of quoted-strings.
  *
- * Return value: (transfer full) (element-type utf8): a #GSList of
+ * Returns: (transfer full) (element-type utf8): a #GSList of
  * list elements, as allocated strings
  **/
 GSList *
@@ -526,7 +535,7 @@ sort_by_qval (const void *a, const void *b)
  * items with qvalue 0. Either way, those items will be removed from
  * the main list.
  *
- * Return value: (transfer full) (element-type utf8): a #GSList of
+ * Returns: (transfer full) (element-type utf8): a #GSList of
  * acceptable values (as allocated strings), highest-qvalue first.
  **/
 GSList *
@@ -624,7 +633,7 @@ soup_header_free_list (GSList *list)
  * case-insensitively). Note that this can't be used with lists
  * that have qvalues.
  *
- * Return value: whether or not @header contains @token
+ * Returns: whether or not @header contains @token
  **/
 gboolean
 soup_header_contains (const char *header, const char *token)
@@ -686,7 +695,7 @@ decode_rfc5987 (char *encoded_string)
 	if (!q)
 		return FALSE;
 
-	decoded = soup_uri_decode (q + 1);
+	decoded = g_uri_unescape_string (q + 1, NULL);
 	if (iso_8859_1) {
 		char *utf8 =  g_convert_with_fallback (decoded, -1, "UTF-8",
 						       "iso-8859-1", "_",
@@ -782,7 +791,7 @@ parse_param_list (const char *header, char delim, gboolean strict)
  * for giving UTF8-encoded filenames in the Content-Disposition
  * header).
  *
- * Return value: (element-type utf8 utf8) (transfer full): a
+ * Returns: (element-type utf8 utf8) (transfer full): a
  * #GHashTable of list elements, which can be freed with
  * soup_header_free_param_list().
  **/
@@ -808,11 +817,10 @@ soup_header_parse_param_list (const char *header)
  * for giving UTF8-encoded filenames in the Content-Disposition
  * header).
  *
- * Return value: (element-type utf8 utf8) (transfer full): a
+ * Returns: (element-type utf8 utf8) (transfer full): a
  * #GHashTable of list elements, which can be freed with
  * soup_header_free_param_list().
  *
- * Since: 2.24
  **/
 GHashTable *
 soup_header_parse_semi_param_list (const char *header)
@@ -834,12 +842,11 @@ soup_header_parse_semi_param_list (const char *header)
  * RFC5987-encoded parameters, use
  * soup_header_parse_param_list() instead.
  *
- * Return value: (element-type utf8 utf8) (transfer full) (nullable):
+ * Returns: (element-type utf8 utf8) (transfer full) (nullable):
  * a #GHashTable of list elements, which can be freed with
  * soup_header_free_param_list() or %NULL if there are duplicate
  * elements.
  *
- * Since: 2.66
  **/
 GHashTable *
 soup_header_parse_param_list_strict (const char *header)
@@ -861,12 +868,11 @@ soup_header_parse_param_list_strict (const char *header)
  * RFC5987-encoded parameters, use
  * soup_header_parse_semi_param_list() instead.
  *
- * Return value: (element-type utf8 utf8) (transfer full) (nullable):
+ * Returns: (element-type utf8 utf8) (transfer full) (nullable):
  * a #GHashTable of list elements, which can be freed with
  * soup_header_free_param_list() or %NULL if there are duplicate
  * elements.
  *
- * Since: 2.66
  **/
 GHashTable *
 soup_header_parse_semi_param_list_strict (const char *header)
@@ -900,7 +906,7 @@ append_param_rfc5987 (GString    *string,
 
 	g_string_append (string, name);
 	g_string_append (string, "*=UTF-8''");
-	encoded = soup_uri_encode (value, " *'%()<>@,;:\\\"/[]?=");
+	encoded = g_uri_escape_string (value, "*'%()<>@,;:\\\"/[]?=", FALSE);
 	g_string_append (string, encoded);
 	g_free (encoded);
 }
@@ -968,7 +974,6 @@ append_param_internal (GString    *string,
  * If @value is (non-ASCII) UTF-8, this will instead use RFC 5987
  * encoding, just like soup_header_g_string_append_param().
  *
- * Since: 2.30
  **/
 void
 soup_header_g_string_append_param_quoted (GString    *string,
@@ -1000,7 +1005,6 @@ soup_header_g_string_append_param_quoted (GString    *string,
  *
  * If @value is %NULL, this will just append @name to @string.
  *
- * Since: 2.26
  **/
 void
 soup_header_g_string_append_param (GString    *string,
