@@ -17,7 +17,9 @@
 #include "soup-logger.h"
 #include "soup-connection.h"
 #include "soup-message-private.h"
+#include "soup-misc.h"
 #include "soup.h"
+#include "soup-session-feature-private.h"
 
 /**
  * SECTION:soup-logger
@@ -40,13 +42,11 @@
  * <informalexample><screen>
  * > POST /unauth HTTP/1.1
  * > Soup-Debug-Timestamp: 1200171744
- * > Soup-Debug: SoupSessionAsync 1 (0x612190), SoupMessage 1 (0x617000), SoupSocket 1 (0x612220)
+ * > Soup-Debug: SoupSession 1 (0x612190), SoupMessage 1 (0x617000), GSocket 1 (0x612220)
  * > Host: localhost
  * > Content-Type: text/plain
  * > Connection: close
- * > 
- * > This is a test.
- *   
+ *
  * &lt; HTTP/1.1 201 Created
  * &lt; Soup-Debug-Timestamp: 1200171744
  * &lt; Soup-Debug: SoupMessage 1 (0x617000)
@@ -59,7 +59,7 @@
  * received.
  *
  * The <literal>Soup-Debug</literal> line gives further debugging
- * information about the #SoupSession, #SoupMessage, and #SoupSocket
+ * information about the #SoupSession, #SoupMessage, and #GSocket
  * involved; the hex numbers are the addresses of the objects in
  * question (which may be useful if you are running in a debugger).
  * The decimal IDs are simply counters that uniquely identify objects
@@ -69,25 +69,33 @@
  *
  * Currently, the request half of the message is logged just before
  * the first byte of the request gets written to the network (from the
- * #SoupMessage::starting signal), which means that if you have
- * not made the complete request body available at that point, it will
- * not be logged.
+ * #SoupMessage::starting signal).
  *
  * The response is logged just after the last byte of the response
- * body is read from the network (from the #SoupMessage::got_body or
- * #SoupMessage::got_informational signal), which means that the
- * #SoupMessage::got_headers signal, and anything triggered off it
- * (such as #SoupSession::authenticate) will be emitted
+ * body is read from the network (from the #SoupMessage::got-body or
+ * #SoupMessage::got-informational signal), which means that the
+ * #SoupMessage::got-headers signal, and anything triggered off it
+ * (such as #SoupMessage::authenticate) will be emitted
  * <emphasis>before</emphasis> the response headers are actually
  * logged.
  *
  * If the response doesn't happen to trigger the
- * #SoupMessage::got_body nor #SoupMessage::got_informational signals
+ * #SoupMessage::got-body nor #SoupMessage::got-informational signals
  * due to, for example, a cancellation before receiving the last byte
  * of the response body, the response will still be logged on the
  * event of the #SoupMessage::finished signal.
  **/
 
+/**
+ * SoupLogger:
+ *
+ * Class implementing logging.
+ */
+
+struct _SoupLogger {
+	GObject parent;
+};
+
 typedef struct {
 	/* We use a mutex so that if requests are being run in
 	 * multiple threads, we don't mix up the output.
@@ -99,7 +107,6 @@ typedef struct {
 
 	SoupSession        *session;
 	SoupLoggerLogLevel  level;
-	int                 max_body_size;
 
 	SoupLoggerFilter    request_filter;
 	gpointer            request_filter_data;
@@ -118,12 +125,10 @@ enum {
 	PROP_0,
 
 	PROP_LEVEL,
-	PROP_MAX_BODY_SIZE,
 
 	LAST_PROP
 };
 
-static SoupSessionFeatureInterface *soup_logger_default_feature_interface;
 static void soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface, gpointer interface_data);
 
 G_DEFINE_TYPE_WITH_CODE (SoupLogger, soup_logger, G_TYPE_OBJECT,
@@ -135,9 +140,12 @@ static void
 soup_logger_init (SoupLogger *logger)
 {
 	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
+	char *id;
 
 	g_mutex_init (&priv->lock);
-	priv->tag = g_quark_from_static_string (g_strdup_printf ("SoupLogger-%p", logger));
+	id = g_strdup_printf ("SoupLogger-%p", logger);
+	priv->tag = g_quark_from_string (id);
+	g_free (id);
 	priv->ids = g_hash_table_new (NULL, NULL);
 }
 
@@ -172,9 +180,6 @@ soup_logger_set_property (GObject *object, guint prop_id,
 	case PROP_LEVEL:
 		priv->level = g_value_get_enum (value);
 		break;
-	case PROP_MAX_BODY_SIZE:
-		priv->max_body_size = g_value_get_int (value);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -192,9 +197,6 @@ soup_logger_get_property (GObject *object, guint prop_id,
 	case PROP_LEVEL:
 		g_value_set_enum (value, priv->level);
 		break;
-	case PROP_MAX_BODY_SIZE:
-		g_value_set_int (value, priv->max_body_size);
-		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 		break;
@@ -216,49 +218,16 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
 	 *
 	 * The level of logging output
 	 *
-	 * Since: 2.56
 	 */
-	/**
-	 * SOUP_LOGGER_LEVEL:
-	 *
-	 * Alias for the #SoupLogger:level property, qv.
-	 *
-	 * Since: 2.56
-	 **/
 	g_object_class_install_property (
 		object_class, PROP_LEVEL,
-		g_param_spec_enum (SOUP_LOGGER_LEVEL,
+		g_param_spec_enum ("level",
 				    "Level",
 				    "The level of logging output",
 				    SOUP_TYPE_LOGGER_LOG_LEVEL,
 				    SOUP_LOGGER_LOG_MINIMAL,
-				    G_PARAM_READWRITE));
-
-	/**
-	 * SoupLogger:max-body-size:
-	 *
-	 * If #SoupLogger:level is %SOUP_LOGGER_LOG_BODY, this gives
-	 * the maximum number of bytes of the body that will be logged.
-	 * (-1 means "no limit".)
-	 *
-	 * Since: 2.56
-	 */
-	/**
-	 * SOUP_LOGGER_MAX_BODY_SIZE:
-	 *
-	 * Alias for the #SoupLogger:max-body-size property, qv.
-	 *
-	 * Since: 2.56
-	 **/
-	g_object_class_install_property (
-		object_class, PROP_MAX_BODY_SIZE,
-		g_param_spec_int (SOUP_LOGGER_MAX_BODY_SIZE,
-				    "Max Body Size",
-				    "The maximum body size to output",
-				    -1,
-				    G_MAXINT,
-				    -1,
-				    G_PARAM_READWRITE));
+				    G_PARAM_READWRITE |
+				    G_PARAM_STATIC_STRINGS));
 }
 
 /**
@@ -267,8 +236,6 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
  * @SOUP_LOGGER_LOG_MINIMAL: Log the Request-Line or Status-Line and
  * the Soup-Debug pseudo-headers
  * @SOUP_LOGGER_LOG_HEADERS: Log the full request/response headers
- * @SOUP_LOGGER_LOG_BODY: Log the full headers and request/response
- * bodies.
  *
  * Describes the level of logging output to provide.
  **/
@@ -276,11 +243,8 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
 /**
  * soup_logger_new:
  * @level: the debug level
- * @max_body_size: the maximum body size to output, or -1
  *
- * Creates a new #SoupLogger with the given debug level. If @level is
- * %SOUP_LOGGER_LOG_BODY, @max_body_size gives the maximum number of
- * bytes of the body that will be logged. (-1 means "no limit".)
+ * Creates a new #SoupLogger with the given debug level.
  *
  * If you need finer control over what message parts are and aren't
  * logged, use soup_logger_set_request_filter() and
@@ -289,12 +253,9 @@ soup_logger_class_init (SoupLoggerClass *logger_class)
  * Returns: a new #SoupLogger
  **/
 SoupLogger *
-soup_logger_new (SoupLoggerLogLevel level, int max_body_size) 
+soup_logger_new (SoupLoggerLogLevel level)
 {
-	return g_object_new (SOUP_TYPE_LOGGER,
-			     SOUP_LOGGER_LEVEL, level,
-			     SOUP_LOGGER_MAX_BODY_SIZE, max_body_size,
-			     NULL);
+	return g_object_new (SOUP_TYPE_LOGGER, "level", level, NULL);
 }
 
 /**
@@ -307,10 +268,9 @@ soup_logger_new (SoupLoggerLogLevel level, int max_body_size)
  * The prototype for a logging filter. The filter callback will be
  * invoked for each request or response, and should analyze it and
  * return a #SoupLoggerLogLevel value indicating how much of the
- * message to log. Eg, it might choose between %SOUP_LOGGER_LOG_BODY
- * and %SOUP_LOGGER_LOG_HEADERS depending on the Content-Type.
+ * message to log.
  *
- * Return value: a #SoupLoggerLogLevel value indicating how much of
+ * Returns: a #SoupLoggerLogLevel value indicating how much of
  * the message to log
  **/
 
@@ -435,43 +395,6 @@ soup_logger_set_id (SoupLogger *logger, gpointer object)
 	return GPOINTER_TO_UINT (id);
 }
 
-/**
- * soup_logger_attach:
- * @logger: a #SoupLogger
- * @session: a #SoupSession
- *
- * Sets @logger to watch @session and print debug information for
- * its messages.
- *
- * (The session will take a reference on @logger, which will be
- * removed when you call soup_logger_detach(), or when the session is
- * destroyed.)
- *
- * Deprecated: Use soup_session_add_feature() instead.
- **/
-void
-soup_logger_attach (SoupLogger  *logger,
-		    SoupSession *session)
-{
-	soup_session_add_feature (session, SOUP_SESSION_FEATURE (logger));
-}
-
-/**
- * soup_logger_detach:
- * @logger: a #SoupLogger
- * @session: a #SoupSession
- *
- * Stops @logger from watching @session.
- *
- * Deprecated: Use soup_session_remove_feature() instead.
- **/
-void
-soup_logger_detach (SoupLogger  *logger,
-		    SoupSession *session)
-{
-	soup_session_remove_feature (session, SOUP_SESSION_FEATURE (logger));
-}
-
 static void soup_logger_print (SoupLogger *logger, SoupLoggerLogLevel level,
 			       char direction, const char *format, ...) G_GNUC_PRINTF (4, 5);
 
@@ -487,11 +410,6 @@ soup_logger_print (SoupLogger *logger, SoupLoggerLogLevel level,
 	data = g_strdup_vprintf (format, args);
 	va_end (args);
 
-	if (level == SOUP_LOGGER_LOG_BODY && priv->max_body_size > 0) {
-		if (strlen (data) > priv->max_body_size + 6)
-			strcpy (data + priv->max_body_size, "\n[...]");
-	}
-
 	line = data;
 	do {
 		end = strchr (line, '\n');
@@ -541,14 +459,14 @@ soup_logger_print_basic_auth (SoupLogger *logger, const char *value)
 
 static void
 print_request (SoupLogger *logger, SoupMessage *msg,
-	       SoupSocket *socket, gboolean restarted)
+	       GSocket *socket, gboolean restarted)
 {
 	SoupLoggerPrivate *priv = soup_logger_get_instance_private (logger);
 	SoupLoggerLogLevel log_level;
 	SoupMessageHeadersIter iter;
 	const char *name, *value;
 	char *socket_dbg;
-	SoupURI *uri;
+	GUri *uri;
 
 	if (priv->request_filter) {
 		log_level = priv->request_filter (logger, msg,
@@ -560,17 +478,18 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 		return;
 
 	uri = soup_message_get_uri (msg);
-	if (msg->method == SOUP_METHOD_CONNECT) {
+	if (soup_message_get_method (msg) == SOUP_METHOD_CONNECT) {
 		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '>',
 				   "CONNECT %s:%u HTTP/1.%d",
-				   uri->host, uri->port,
+				   g_uri_get_host (uri), g_uri_get_port (uri),
 				   soup_message_get_http_version (msg));
 	} else {
 		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '>',
 				   "%s %s%s%s HTTP/1.%d",
-				   msg->method, uri->path,
-				   uri->query ? "?" : "",
-				   uri->query ? uri->query : "",
+				   soup_message_get_method (msg),
+                                   g_uri_get_path (uri),
+				   g_uri_get_query (uri) ? "?" : "",
+				   g_uri_get_query (uri) ? g_uri_get_query (uri) : "",
 				   soup_message_get_http_version (msg));
 	}
 
@@ -597,25 +516,7 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
 		return;
 
-	if (!soup_message_headers_get_one (msg->request_headers, "Host")) {
-		char *uri_host;
-
-		if (strchr (uri->host, ':'))
-			uri_host = g_strdup_printf ("[%s]", uri->host);
-		else if (g_hostname_is_non_ascii (uri->host))
-			uri_host = g_hostname_to_ascii (uri->host);
-		else
-			uri_host = uri->host;
-
-		soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '>',
-				   "Host: %s%c%u", uri_host,
-				   soup_uri_uses_default_port (uri) ? '\0' : ':',
-				   uri->port);
-
-		if (uri_host != uri->host)
-			g_free (uri_host);
-	}
-	soup_message_headers_iter_init (&iter, msg->request_headers);
+	soup_message_headers_iter_init (&iter, soup_message_get_request_headers (msg));
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
 		if (!g_ascii_strcasecmp (name, "Authorization") &&
 		    !g_ascii_strncasecmp (value, "Basic ", 6))
@@ -625,22 +526,6 @@ print_request (SoupLogger *logger, SoupMessage *msg,
 					   "%s: %s", name, value);
 		}
 	}
-	if (log_level == SOUP_LOGGER_LOG_HEADERS)
-		return;
-
-	if (msg->request_body->length &&
-	    soup_message_body_get_accumulate (msg->request_body)) {
-		SoupBuffer *request;
-
-		request = soup_message_body_flatten (msg->request_body);
-		g_return_if_fail (request != NULL);
-		soup_buffer_free (request);
-
-		if (soup_message_headers_get_expectations (msg->request_headers) != SOUP_EXPECTATION_CONTINUE) {
-			soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '>',
-					   "\n%s", msg->request_body->data);
-		}
-	}
 }
 
 static void
@@ -663,7 +548,7 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '<',
 			   "HTTP/1.%d %u %s\n",
 			   soup_message_get_http_version (msg),
-			   msg->status_code, msg->reason_phrase);
+			   soup_message_get_status (msg), soup_message_get_reason_phrase (msg));
 
 	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '<',
 			   "Soup-Debug-Timestamp: %lu",
@@ -676,18 +561,11 @@ print_response (SoupLogger *logger, SoupMessage *msg)
 	if (log_level == SOUP_LOGGER_LOG_MINIMAL)
 		return;
 
-	soup_message_headers_iter_init (&iter, msg->response_headers);
+	soup_message_headers_iter_init (&iter, soup_message_get_response_headers (msg));
 	while (soup_message_headers_iter_next (&iter, &name, &value)) {
 		soup_logger_print (logger, SOUP_LOGGER_LOG_HEADERS, '<',
 				   "%s: %s", name, value);
 	}
-	if (log_level == SOUP_LOGGER_LOG_HEADERS)
-		return;
-
-	if (msg->response_body->data) {
-		soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '<',
-				   "\n%s", msg->response_body->data);
-	}
 }
 
 static void
@@ -699,7 +577,7 @@ finished (SoupMessage *msg, gpointer user_data)
 	g_mutex_lock (&priv->lock);
 
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 
 	g_mutex_unlock (&priv->lock);
 }
@@ -714,26 +592,12 @@ got_informational (SoupMessage *msg, gpointer user_data)
 
 	g_signal_handlers_disconnect_by_func (msg, finished, logger);
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
-
-	if (msg->status_code == SOUP_STATUS_CONTINUE && msg->request_body->data) {
-		SoupLoggerLogLevel log_level;
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 
+	if (soup_message_get_status (msg) == SOUP_STATUS_CONTINUE && soup_message_get_request_body_stream (msg)) {
 		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, '>',
 				   "[Now sending request body...]");
-
-		if (priv->request_filter) {
-			log_level = priv->request_filter (logger, msg,
-							  priv->request_filter_data);
-		} else
-			log_level = priv->level;
-
-		if (log_level == SOUP_LOGGER_LOG_BODY) {
-			soup_logger_print (logger, SOUP_LOGGER_LOG_BODY, '>',
-					   "%s", msg->request_body->data);
-		}
-
-		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
+		soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 	}
 
 	g_mutex_unlock (&priv->lock);
@@ -749,7 +613,7 @@ got_body (SoupMessage *msg, gpointer user_data)
 
 	g_signal_handlers_disconnect_by_func (msg, finished, logger);
 	print_response (logger, msg);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 
 	g_mutex_unlock (&priv->lock);
 }
@@ -762,7 +626,7 @@ starting (SoupMessage *msg, gpointer user_data)
 	gboolean restarted;
 	guint msg_id;
 	SoupConnection *conn;
-	SoupSocket *socket;
+	GSocket *socket;
 
 	msg_id = soup_logger_get_id (logger, msg);
 	if (msg_id)
@@ -781,19 +645,18 @@ starting (SoupMessage *msg, gpointer user_data)
 		soup_logger_set_id (logger, socket);
 
 	print_request (logger, msg, socket, restarted);
-	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "");
+	soup_logger_print (logger, SOUP_LOGGER_LOG_MINIMAL, ' ', "\n");
 }
 
 static void
 soup_logger_request_queued (SoupSessionFeature *logger,
-			    SoupSession *session,
-			    SoupMessage *msg)
+			    SoupMessage        *msg)
 {
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 
-	g_signal_connect (msg, "starting",
-			  G_CALLBACK (starting),
-			  logger);
+	g_signal_connect_after (msg, "starting",
+				G_CALLBACK (starting),
+				logger);
 	g_signal_connect (msg, "got-informational",
 			  G_CALLBACK (got_informational),
 			  logger);
@@ -807,15 +670,11 @@ soup_logger_request_queued (SoupSessionFeature *logger,
 
 static void
 soup_logger_request_unqueued (SoupSessionFeature *logger,
-			      SoupSession *session,
-			      SoupMessage *msg)
+			      SoupMessage        *msg)
 {
 	g_return_if_fail (SOUP_IS_MESSAGE (msg));
 
-	g_signal_handlers_disconnect_by_func (msg, starting, logger);
-	g_signal_handlers_disconnect_by_func (msg, got_informational, logger);
-	g_signal_handlers_disconnect_by_func (msg, got_body, logger);
-	g_signal_handlers_disconnect_by_func (msg, finished, logger);
+	g_signal_handlers_disconnect_by_data (msg, logger);
 }
 
 static void
@@ -825,17 +684,12 @@ soup_logger_feature_attach (SoupSessionFeature *feature,
 	SoupLoggerPrivate *priv = soup_logger_get_instance_private (SOUP_LOGGER (feature));
 
 	priv->session = session;
-
-	soup_logger_default_feature_interface->attach (feature, session);
 }
 
 static void
 soup_logger_session_feature_init (SoupSessionFeatureInterface *feature_interface,
 				  gpointer interface_data)
 {
-	soup_logger_default_feature_interface =
-		g_type_default_interface_peek (SOUP_TYPE_SESSION_FEATURE);
-
 	feature_interface->attach = soup_logger_feature_attach;
 	feature_interface->request_queued = soup_logger_request_queued;
 	feature_interface->request_unqueued = soup_logger_request_unqueued;
