@@ -13,7 +13,7 @@
 
 #include "soup-message-headers.h"
 #include "soup.h"
-#include "soup-misc-private.h"
+#include "soup-misc.h"
 
 /**
  * SECTION:soup-message-headers
@@ -49,7 +49,7 @@ typedef struct {
 	char *value;
 } SoupHeader;
 
-struct SoupMessageHeaders {
+struct _SoupMessageHeaders {
 	GArray *array;
 	GHashTable *concat;
 	SoupMessageHeadersType type;
@@ -59,7 +59,7 @@ struct SoupMessageHeaders {
 	SoupExpectation expectations;
 	char *content_type;
 
-	int ref_count;
+	gatomicrefcount ref_count;
 };
 
 /**
@@ -70,7 +70,7 @@ struct SoupMessageHeaders {
  * automatically for its own headers. You would only need to use this
  * method if you are manually parsing or generating message headers.)
  *
- * Return value: a new #SoupMessageHeaders
+ * Returns: a new #SoupMessageHeaders
  **/
 SoupMessageHeaders *
 soup_message_headers_new (SoupMessageHeadersType type)
@@ -82,7 +82,8 @@ soup_message_headers_new (SoupMessageHeadersType type)
 	hdrs->array = g_array_sized_new (TRUE, FALSE, sizeof (SoupHeader), 5);
 	hdrs->type = type;
 	hdrs->encoding = -1;
-	hdrs->ref_count = 1;
+
+        g_atomic_ref_count_init (&hdrs->ref_count);
 
 	return hdrs;
 }
@@ -90,7 +91,7 @@ soup_message_headers_new (SoupMessageHeadersType type)
 static SoupMessageHeaders *
 soup_message_headers_copy (SoupMessageHeaders *hdrs)
 {
-	g_atomic_int_inc (&hdrs->ref_count);
+	g_atomic_ref_count_inc (&hdrs->ref_count);
 	return hdrs;
 }
 
@@ -103,7 +104,7 @@ soup_message_headers_copy (SoupMessageHeaders *hdrs)
 void
 soup_message_headers_free (SoupMessageHeaders *hdrs)
 {
-	if (!g_atomic_int_dec_and_test (&hdrs->ref_count))
+	if (!g_atomic_ref_count_dec (&hdrs->ref_count))
 		return;
 
 	soup_message_headers_clear (hdrs);
@@ -120,9 +121,8 @@ G_DEFINE_BOXED_TYPE (SoupMessageHeaders, soup_message_headers, soup_message_head
  *
  * Gets the type of headers.
  *
- * Return value: the header's type.
+ * Returns: the header's type.
  *
- * Since: 2.50
  **/
 SoupMessageHeadersType
 soup_message_headers_get_headers_type (SoupMessageHeaders *hdrs)
@@ -158,7 +158,6 @@ soup_message_headers_clear (SoupMessageHeaders *hdrs)
  *
  * Removes all the headers listed in the Connection header.
  *
- * Since: 2.36
  */
 void
 soup_message_headers_clean_connection_headers (SoupMessageHeaders *hdrs)
@@ -296,7 +295,9 @@ soup_message_headers_remove (SoupMessageHeaders *hdrs, const char *name)
 
 	name = intern_header_name (name, &setter);
 	while ((index = find_header (hdr_array, name, 0)) != -1) {
+#ifndef __clang_analyzer__ /* False positive for double-free */
 		g_free (hdr_array[index].value);
+#endif
 		g_array_remove_index (hdrs->array, index);
 	}
 	if (hdrs->concat)
@@ -320,9 +321,8 @@ soup_message_headers_remove (SoupMessageHeaders *hdrs, const char *name)
  * whichever one makes libsoup most compatible with other HTTP
  * implementations.)
  *
- * Return value: (nullable) (transfer none): the header's value or %NULL if not found.
+ * Returns: (nullable) (transfer none): the header's value or %NULL if not found.
  *
- * Since: 2.28
  **/
 const char *
 soup_message_headers_get_one (SoupMessageHeaders *hdrs, const char *name)
@@ -352,10 +352,9 @@ soup_message_headers_get_one (SoupMessageHeaders *hdrs, const char *name)
  * (If @name is present in @hdrs, then this is equivalent to calling
  * soup_header_contains() on its value.)
  *
- * Return value: %TRUE if the header is present and contains @token,
+ * Returns: %TRUE if the header is present and contains @token,
  *   %FALSE otherwise.
  *
- * Since: 2.50
  **/
 gboolean
 soup_message_headers_header_contains (SoupMessageHeaders *hdrs, const char *name, const char *token)
@@ -377,10 +376,9 @@ soup_message_headers_header_contains (SoupMessageHeaders *hdrs, const char *name
  * Checks whether the header @name is present in @hdrs and is
  * (case-insensitively) equal to @value.
  *
- * Return value: %TRUE if the header is present and its value is
+ * Returns: %TRUE if the header is present and its value is
  *   @value, %FALSE otherwise.
  *
- * Since: 2.50
  **/
 gboolean
 soup_message_headers_header_equals (SoupMessageHeaders *hdrs, const char *name, const char *value)
@@ -411,9 +409,8 @@ soup_message_headers_header_equals (SoupMessageHeaders *hdrs, const char *name,
  * transformation is allowed, and so an upstream proxy could do the
  * same thing.
  * 
- * Return value: (nullable) (transfer none): the header's value or %NULL if not found.
+ * Returns: (nullable) (transfer none): the header's value or %NULL if not found.
  *
- * Since: 2.28
  **/
 const char *
 soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
@@ -452,32 +449,6 @@ soup_message_headers_get_list (SoupMessageHeaders *hdrs, const char *name)
 	return value;
 }
 
-/**
- * soup_message_headers_get:
- * @hdrs: a #SoupMessageHeaders
- * @name: header name
- * 
- * Gets the value of header @name in @hdrs.
- *
- * This method was supposed to work correctly for both single-valued
- * and list-valued headers, but because some HTTP clients/servers
- * mistakenly send multiple copies of headers that are supposed to be
- * single-valued, it sometimes returns incorrect results. To fix this,
- * the methods soup_message_headers_get_one() and
- * soup_message_headers_get_list() were introduced, so callers can
- * explicitly state which behavior they are expecting.
- *
- * Return value: (nullable): as with soup_message_headers_get_list().
- * 
- * Deprecated: Use soup_message_headers_get_one() or
- * soup_message_headers_get_list() instead.
- **/
-const char *
-soup_message_headers_get (SoupMessageHeaders *hdrs, const char *name)
-{
-	return soup_message_headers_get_list (hdrs, name);
-}
-
 /**
  * SoupMessageHeadersIter:
  *
@@ -527,7 +498,7 @@ soup_message_headers_iter_init (SoupMessageHeadersIter *iter,
  * then soup_message_headers_iter_next() will return %FALSE and @name
  * and @value will be unchanged.
  *
- * Return value: %TRUE if another name and value were returned, %FALSE
+ * Returns: %TRUE if another name and value were returned, %FALSE
  * if the end of the headers has been reached.
  **/
 gboolean
@@ -563,7 +534,7 @@ soup_message_headers_iter_next (SoupMessageHeadersIter *iter,
  * 
  * Calls @func once for each header value in @hdrs.
  *
- * Beware that unlike soup_message_headers_get(), this processes the
+ * Beware that unlike soup_message_headers_get_list(), this processes the
  * headers in exactly the way they were added, rather than
  * concatenating multiple same-named headers into a single value.
  * (This is intentional; it ensures that if you call
@@ -664,9 +635,15 @@ static void
 transfer_encoding_setter (SoupMessageHeaders *hdrs, const char *value)
 {
 	if (value) {
+		/* "identity" is a wrong value according to RFC errata 408,
+		 * and RFC 7230 does not list it as valid transfer-coding.
+		 * Nevertheless, the obsolete RFC 2616 stated "identity"
+		 * as valid, so we can't handle it as unrecognized here
+		 * for compatibility reasons.
+		 */
 		if (g_ascii_strcasecmp (value, "chunked") == 0)
 			hdrs->encoding = SOUP_ENCODING_CHUNKED;
-		else
+		else if (g_ascii_strcasecmp (value, "identity") != 0)
 			hdrs->encoding = SOUP_ENCODING_UNRECOGNIZED;
 	} else
 		hdrs->encoding = -1;
@@ -715,7 +692,7 @@ content_length_setter (SoupMessageHeaders *hdrs, const char *value)
  * response may declare a Content-Length or Transfer-Encoding, but
  * it will never actually include a body.
  *
- * Return value: the encoding declared by @hdrs.
+ * Returns: the encoding declared by @hdrs.
  **/
 SoupEncoding
 soup_message_headers_get_encoding (SoupMessageHeaders *hdrs)
@@ -794,7 +771,7 @@ soup_message_headers_set_encoding (SoupMessageHeaders *hdrs,
  * be non-0 if soup_message_headers_get_encoding() returns
  * %SOUP_ENCODING_CONTENT_LENGTH.
  *
- * Return value: the message body length declared by @hdrs.
+ * Returns: the message body length declared by @hdrs.
  **/
 goffset
 soup_message_headers_get_content_length (SoupMessageHeaders *hdrs)
@@ -865,7 +842,7 @@ expectation_setter (SoupMessageHeaders *hdrs, const char *value)
  * Currently this will either be %SOUP_EXPECTATION_CONTINUE or
  * %SOUP_EXPECTATION_UNRECOGNIZED.
  *
- * Return value: the contents of @hdrs's "Expect" header
+ * Returns: the contents of @hdrs's "Expect" header
  **/
 SoupExpectation
 soup_message_headers_get_expectations (SoupMessageHeaders *hdrs)
@@ -921,7 +898,6 @@ soup_message_headers_set_expectations (SoupMessageHeaders *hdrs,
  * range", referring to the last -@start bytes of the resource body.
  * (Eg, the last 500 bytes would be @start = -500 and @end = -1.)
  *
- * Since: 2.26
  **/
 
 static int
@@ -1067,11 +1043,10 @@ soup_message_headers_get_ranges_internal (SoupMessageHeaders  *hdrs,
  * actually requested by the client.
  * </para></note>
  *
- * Return value: %TRUE if @hdrs contained a syntactically-valid
+ * Returns: %TRUE if @hdrs contained a syntactically-valid
  * "Range" header, %FALSE otherwise (in which case @range and @length
  * will not be set).
  *
- * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
@@ -1092,7 +1067,6 @@ soup_message_headers_get_ranges (SoupMessageHeaders  *hdrs,
  *
  * Frees the array of ranges returned from soup_message_headers_get_ranges().
  *
- * Since: 2.26
  **/
 void
 soup_message_headers_free_ranges (SoupMessageHeaders  *hdrs,
@@ -1111,7 +1085,6 @@ soup_message_headers_free_ranges (SoupMessageHeaders  *hdrs,
  * only want to request a single range, you can use
  * soup_message_headers_set_range().)
  *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
@@ -1153,7 +1126,6 @@ soup_message_headers_set_ranges (SoupMessageHeaders  *hdrs,
  * If you need to request multiple ranges, use
  * soup_message_headers_set_ranges().
  *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
@@ -1179,10 +1151,9 @@ soup_message_headers_set_range (SoupMessageHeaders  *hdrs,
  * @end, and @total_length. If the total length field in the header
  * was specified as "*", then @total_length will be set to -1.
  *
- * Return value: %TRUE if @hdrs contained a "Content-Range" header
+ * Returns: %TRUE if @hdrs contained a "Content-Range" header
  * containing a byte range which could be parsed, %FALSE otherwise.
  *
- * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
@@ -1238,7 +1209,6 @@ soup_message_headers_get_content_range (SoupMessageHeaders  *hdrs,
  * soup_message_headers_get_ranges() for more details.
  * </para></note>
  *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_content_range (SoupMessageHeaders  *hdrs,
@@ -1351,12 +1321,11 @@ content_type_setter (SoupMessageHeaders *hdrs, const char *value)
  * its value in *@content_type and *@params. @params can be %NULL if you
  * are only interested in the content type itself.
  *
- * Return value: (nullable): a string with the value of the
+ * Returns: (nullable): a string with the value of the
  * "Content-Type" header or %NULL if @hdrs does not contain that
  * header or it cannot be parsed (in which case *@params will be
  * unchanged).
  *
- * Since: 2.26
  **/
 const char *
 soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
@@ -1380,7 +1349,6 @@ soup_message_headers_get_content_type (SoupMessageHeaders  *hdrs,
  * Sets the "Content-Type" header in @hdrs to @content_type,
  * optionally with additional parameters specified in @params.
  *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_content_type (SoupMessageHeaders  *hdrs,
@@ -1415,11 +1383,10 @@ soup_message_headers_set_content_type (SoupMessageHeaders  *hdrs,
  * this is handled automatically by #SoupMultipart and the associated
  * form methods.
  *
- * Return value: %TRUE if @hdrs contains a "Content-Disposition"
+ * Returns: %TRUE if @hdrs contains a "Content-Disposition"
  * header, %FALSE if not (in which case *@disposition and *@params
  * will be unchanged).
  *
- * Since: 2.26
  **/
 gboolean
 soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
@@ -1458,7 +1425,6 @@ soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
  * See soup_message_headers_get_content_disposition() for a discussion
  * of how Content-Disposition is used in HTTP.
  *
- * Since: 2.26
  **/
 void
 soup_message_headers_set_content_disposition (SoupMessageHeaders  *hdrs,
