@@ -3,14 +3,14 @@
  * Copyright (C) 2005 Novell, Inc.
  */
 
-#ifndef SOUP_MESSAGE_HEADERS_H
-#define SOUP_MESSAGE_HEADERS_H 1
+#pragma once
 
-#include <libsoup/soup-types.h>
+#include "soup-types.h"
 
 G_BEGIN_DECLS
 
-typedef struct SoupMessageHeaders SoupMessageHeaders;
+typedef struct _SoupMessageHeaders SoupMessageHeaders;
+SOUP_AVAILABLE_IN_ALL
 GType soup_message_headers_get_type (void);
 #define SOUP_TYPE_MESSAGE_HEADERS (soup_message_headers_get_type ())
 
@@ -20,40 +20,41 @@ typedef enum {
 	SOUP_MESSAGE_HEADERS_MULTIPART
 } SoupMessageHeadersType;
 
+SOUP_AVAILABLE_IN_ALL
 SoupMessageHeaders *soup_message_headers_new      (SoupMessageHeadersType type);
 
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_free     (SoupMessageHeaders *hdrs);
 
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_append   (SoupMessageHeaders *hdrs,
 						   const char         *name,
 						   const char         *value);
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_replace  (SoupMessageHeaders *hdrs,
 						   const char         *name,
 						   const char         *value);
 
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_remove   (SoupMessageHeaders *hdrs,
 						   const char         *name);
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_clear    (SoupMessageHeaders *hdrs);
 
-SOUP_AVAILABLE_IN_2_36
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_clean_connection_headers (SoupMessageHeaders *hdrs);
 
-#ifndef SOUP_DISABLE_DEPRECATED
-SOUP_DEPRECATED_IN_2_28_FOR ("soup_message_headers_get_one or soup_message_headers_get_list")
-const char         *soup_message_headers_get      (SoupMessageHeaders *hdrs,
-						   const char         *name);
-#endif
-SOUP_AVAILABLE_IN_2_28
+SOUP_AVAILABLE_IN_ALL
 const char         *soup_message_headers_get_one  (SoupMessageHeaders *hdrs,
 						   const char         *name);
-SOUP_AVAILABLE_IN_2_28
+SOUP_AVAILABLE_IN_ALL
 const char         *soup_message_headers_get_list (SoupMessageHeaders *hdrs,
 						   const char         *name);
-SOUP_AVAILABLE_IN_2_50
+SOUP_AVAILABLE_IN_ALL
 gboolean            soup_message_headers_header_contains (SoupMessageHeaders *hdrs,
 							  const char         *name,
 							  const char         *token);
-SOUP_AVAILABLE_IN_2_50
+SOUP_AVAILABLE_IN_ALL
 gboolean            soup_message_headers_header_equals   (SoupMessageHeaders *hdrs,
 							  const char         *name,
 							  const char         *value);
@@ -62,11 +63,12 @@ typedef void      (*SoupMessageHeadersForeachFunc)(const char         *name,
 						   const char         *value,
 						   gpointer            user_data);
 
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_foreach  (SoupMessageHeaders *hdrs,
 						   SoupMessageHeadersForeachFunc func,
 						   gpointer            user_data);
 
-SOUP_AVAILABLE_IN_2_50
+SOUP_AVAILABLE_IN_ALL
 SoupMessageHeadersType soup_message_headers_get_headers_type (SoupMessageHeaders *hdrs);
 
 typedef struct {
@@ -74,8 +76,10 @@ typedef struct {
 	gpointer dummy[3];
 } SoupMessageHeadersIter;
 
+SOUP_AVAILABLE_IN_ALL
 void                soup_message_headers_iter_init (SoupMessageHeadersIter  *iter,
 						    SoupMessageHeaders      *hdrs);
+SOUP_AVAILABLE_IN_ALL
 gboolean            soup_message_headers_iter_next (SoupMessageHeadersIter  *iter,
 						    const char             **name,
 						    const char             **value);
@@ -91,11 +95,15 @@ typedef enum {
 	SOUP_ENCODING_BYTERANGES
 } SoupEncoding;
 
+SOUP_AVAILABLE_IN_ALL
 SoupEncoding    soup_message_headers_get_encoding        (SoupMessageHeaders *hdrs);
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_set_encoding        (SoupMessageHeaders *hdrs,
 							  SoupEncoding        encoding);
 
+SOUP_AVAILABLE_IN_ALL
 goffset         soup_message_headers_get_content_length  (SoupMessageHeaders *hdrs);
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_set_content_length  (SoupMessageHeaders *hdrs,
 							  goffset             content_length);
 
@@ -104,7 +112,9 @@ typedef enum {
 	SOUP_EXPECTATION_CONTINUE     = (1 << 1)
 } SoupExpectation;
 
+SOUP_AVAILABLE_IN_ALL
 SoupExpectation soup_message_headers_get_expectations    (SoupMessageHeaders *hdrs);
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_set_expectations    (SoupMessageHeaders *hdrs,
 							  SoupExpectation     expectations);
 
@@ -113,52 +123,52 @@ typedef struct {
 	goffset end;
 } SoupRange;
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 gboolean        soup_message_headers_get_ranges          (SoupMessageHeaders  *hdrs,
 							  goffset              total_length,
 							  SoupRange          **ranges,
 							  int                 *length);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_free_ranges         (SoupMessageHeaders  *hdrs,
 							  SoupRange           *ranges);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_set_ranges          (SoupMessageHeaders  *hdrs,
 							  SoupRange           *ranges,
 							  int                  length);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_set_range           (SoupMessageHeaders  *hdrs,
 							  goffset              start,
 							  goffset              end);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 gboolean        soup_message_headers_get_content_range   (SoupMessageHeaders  *hdrs,
 							  goffset             *start,
 							  goffset             *end,
 							  goffset             *total_length);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void            soup_message_headers_set_content_range   (SoupMessageHeaders  *hdrs,
 							  goffset              start,
 							  goffset              end,
 							  goffset              total_length);
 
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 const char *soup_message_headers_get_content_type     (SoupMessageHeaders  *hdrs,
 						       GHashTable         **params);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void        soup_message_headers_set_content_type     (SoupMessageHeaders  *hdrs,
 						       const char          *content_type,
 						       GHashTable          *params);
 
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 gboolean soup_message_headers_get_content_disposition (SoupMessageHeaders  *hdrs,
 						       char               **disposition,
 						       GHashTable         **params);
-SOUP_AVAILABLE_IN_2_26
+SOUP_AVAILABLE_IN_ALL
 void     soup_message_headers_set_content_disposition (SoupMessageHeaders  *hdrs,
 						       const char          *disposition,
 						       GHashTable          *params);
 
-G_END_DECLS
+G_DEFINE_AUTOPTR_CLEANUP_FUNC (SoupMessageHeaders, soup_message_headers_free)
 
-#endif /* SOUP_MESSAGE_HEADERS_H */
+G_END_DECLS
