@@ -70,25 +70,28 @@ queue_message_restarted (SoupMessage *msg, gpointer user_data)
  *
  * Creates a new #SoupMessageQueueItem and appends it to @queue.
  *
- * Return value: the new item, which you must unref with
+ * Returns: the new item, which you must unref with
  * soup_message_queue_unref_item() when you are done with.
  **/
 SoupMessageQueueItem *
-soup_message_queue_append (SoupMessageQueue *queue, SoupMessage *msg,
-			   SoupSessionCallback callback, gpointer user_data)
+soup_message_queue_append (SoupMessageQueue   *queue,
+			   SoupMessage        *msg,
+			   GCancellable       *cancellable,
+			   SoupSessionCallback callback,
+			   gpointer            user_data)
 {
 	SoupMessageQueueItem *item;
 
 	item = g_slice_new0 (SoupMessageQueueItem);
 	item->session = g_object_ref (queue->session);
-	item->async_context = soup_session_get_async_context (item->session);
+	item->async_context = g_main_context_get_thread_default ();
 	if (item->async_context)
 		g_main_context_ref (item->async_context);
 	item->queue = queue;
 	item->msg = g_object_ref (msg);
 	item->callback = callback;
 	item->callback_data = user_data;
-	item->cancellable = g_cancellable_new ();
+	item->cancellable = cancellable ? g_object_ref (cancellable) : g_cancellable_new ();
 	item->priority = soup_message_get_priority (msg);
 
 	g_signal_connect (msg, "restarted",
@@ -188,10 +191,6 @@ soup_message_queue_item_unref (SoupMessageQueueItem *item)
 	g_clear_error (&item->error);
 	g_clear_object (&item->task);
 	g_clear_pointer (&item->async_context, g_main_context_unref);
-	if (item->io_source) {
-		g_source_destroy (item->io_source);
-		g_source_unref (item->io_source);
-	}
 	g_slice_free (SoupMessageQueueItem, item);
 }
 
@@ -204,7 +203,7 @@ soup_message_queue_item_unref (SoupMessageQueueItem *item)
  * the item with soup_message_queue_unref_item() when you are done
  * with it.
  *
- * Return value: the queue item for @msg, or %NULL
+ * Returns: (nullable): the queue item for @msg, or %NULL
  **/ 
 SoupMessageQueueItem *
 soup_message_queue_lookup (SoupMessageQueue *queue, SoupMessage *msg)
@@ -234,7 +233,7 @@ soup_message_queue_lookup (SoupMessageQueue *queue, SoupMessage *msg)
  * only need to unref the item yourself if you are not going to
  * finishing walking the queue.)
  *
- * Return value: the first item in @queue.
+ * Returns: the first item in @queue.
  **/ 
 SoupMessageQueueItem *
 soup_message_queue_first (SoupMessageQueue *queue)
@@ -264,7 +263,7 @@ soup_message_queue_first (SoupMessageQueue *queue)
  * yourself with soup_message_queue_unref_item() if you do not finish
  * walking the queue.
  *
- * Return value: the next item in @queue.
+ * Returns: the next item in @queue.
  **/ 
 SoupMessageQueueItem *
 soup_message_queue_next (SoupMessageQueue *queue, SoupMessageQueueItem *item)
