@@ -4,8 +4,8 @@
  * Copyright (C) 2008 Red Hat, Inc.
  */
 
-#ifndef SOUP_MESSAGE_QUEUE_H
-#define SOUP_MESSAGE_QUEUE_H 1
+#ifndef __SOUP_MESSAGE_QUEUE_H__
+#define __SOUP_MESSAGE_QUEUE_H__ 1
 
 #include "soup-connection.h"
 #include "soup-message.h"
@@ -28,6 +28,10 @@ typedef enum {
 	SOUP_MESSAGE_FINISHED
 } SoupMessageQueueItemState;
 
+typedef void (*SoupSessionCallback) (SoupSession *session,
+				     SoupMessage *msg,
+				     gpointer     user_data);
+
 struct _SoupMessageQueueItem {
 	/*< public >*/
 	SoupSession *session;
@@ -42,17 +46,15 @@ struct _SoupMessageQueueItem {
 
 	SoupConnection *conn;
 	GTask *task;
-	GSource *io_source;
 
 	guint paused            : 1;
-	guint new_api           : 1;
 	guint io_started        : 1;
 	guint async             : 1;
 	guint async_pending     : 1;
-	guint conn_is_dedicated : 1;
 	guint connect_only      : 1;
 	guint priority          : 3;
 	guint resend_count      : 5;
+	int io_priority;
 
 	SoupMessageQueueItemState state;
 
@@ -66,6 +68,7 @@ struct _SoupMessageQueueItem {
 SoupMessageQueue     *soup_message_queue_new        (SoupSession          *session);
 SoupMessageQueueItem *soup_message_queue_append     (SoupMessageQueue     *queue,
 						     SoupMessage          *msg,
+						     GCancellable         *cancellable,
 						     SoupSessionCallback   callback,
 						     gpointer              user_data);
 
@@ -86,4 +89,4 @@ void soup_message_queue_item_unref          (SoupMessageQueueItem *item);
 
 G_END_DECLS
 
-#endif /* SOUP_MESSAGE_QUEUE_H */
+#endif /* __SOUP_MESSAGE_QUEUE_H__ */
