@@ -5,96 +5,100 @@
  */
 
 #include "test-utils.h"
+#include "soup-uri-utils-private.h"
 
-SoupURI *http_uri;
-SoupURI *https_uri;
+GUri *http_uri;
+GUri *https_uri;
 
 /* This server pseudo-implements the HSTS spec in order to allow us to
    test the Soup HSTS feature.
  */
 static void
-server_callback  (SoupServer *server, SoupMessage *msg,
-		  const char *path, GHashTable *query,
-		  SoupClientContext *context, gpointer data)
+server_callback  (SoupServer        *server,
+		  SoupServerMessage *msg,
+		  const char        *path,
+		  GHashTable        *query,
+		  gpointer           data)
 {
+	SoupMessageHeaders *response_headers;
 	const char *server_protocol = data;
 
+	response_headers = soup_server_message_get_response_headers (msg);
+
 	if (strcmp (server_protocol, "http") == 0) {
 		if (strcmp (path, "/insecure") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=31536000");
-			soup_message_set_status (msg, SOUP_STATUS_OK);
+			soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
 		} else {
-			char *uri_string;
-			SoupURI *uri = soup_uri_new ("https://localhost");
-			soup_uri_set_path (uri, path);
-			uri_string = soup_uri_to_string (uri, FALSE);
-			soup_message_set_redirect (msg, SOUP_STATUS_MOVED_PERMANENTLY, uri_string);
-			soup_uri_free (uri);
+                        GUri *uri = g_uri_build (SOUP_HTTP_URI_FLAGS, "https", NULL, "localhost", -1, path, NULL, NULL);
+			char *uri_string = g_uri_to_string (uri);
+			soup_server_message_set_redirect (msg, SOUP_STATUS_MOVED_PERMANENTLY, uri_string);
+			g_uri_unref (uri);
 			g_free (uri_string);
 		}
 	} else if (strcmp (server_protocol, "https") == 0) {
-		soup_message_set_status (msg, SOUP_STATUS_OK);
+		soup_server_message_set_status (msg, SOUP_STATUS_OK, NULL);
 		if (strcmp (path, "/long-lasting") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=31536000");
 		} else if (strcmp (path, "/two-seconds") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=2");
 		} else if (strcmp (path, "/three-seconds") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=3");
 		} else if (strcmp (path, "/delete") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=0");
 		} else if (strcmp (path, "/subdomains") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=31536000; includeSubDomains");
 		} else if (strcmp (path, "/no-sts-header") == 0) {
 			/* Do not add anything */
 		} else if (strcmp (path, "/multiple-headers") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=31536000; includeSubDomains");
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=1; includeSubDomains");
 		} else if (strcmp (path, "/missing-values") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "");
 		} else if (strcmp (path, "/invalid-values") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=foo");
 		} else if (strcmp (path, "/extra-values-0") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=3600; foo");
 		} else if (strcmp (path, "/extra-values-1") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     " max-age=3600; includeDomains; foo");
 		} else if (strcmp (path, "/duplicated-directives") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=3600; includeDomains; includeDomains");
 		} else if (strcmp (path, "/case-insensitive-header") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "STRICT-TRANSPORT-SECURITY",
 						     "max-age=3600");
 		} else if (strcmp (path, "/case-insensitive-directives") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "MAX-AGE=3600; includesubdomains");
 		} else if (strcmp (path, "/optional-quotations") == 0) {
-			soup_message_headers_append (msg->response_headers,
+			soup_message_headers_append (response_headers,
 						     "Strict-Transport-Security",
 						     "max-age=\"31536000\"");
 		}
@@ -102,14 +106,35 @@ server_callback  (SoupServer *server, SoupMessage *msg,
 }
 
 static void
-session_get_uri (SoupSession *session, const char *uri, SoupStatus expected_status)
+hsts_enforced_cb (SoupMessage *msg,
+		  gboolean    *enforced)
+{
+	*enforced = TRUE;
+}
+
+static void
+session_get_uri (SoupSession *session,
+		 const char  *uri,
+		 SoupStatus   expected_status,
+		 gboolean     expected_enforced)
 {
 	SoupMessage *msg;
+	GBytes *body;
+	GError *error = NULL;
+	gboolean enforced = FALSE;
 
 	msg = soup_message_new ("GET", uri);
-	soup_message_set_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
-	soup_session_send_message (session, msg);
+	g_signal_connect (msg, "hsts-enforced", G_CALLBACK (hsts_enforced_cb), &enforced);
+	soup_message_add_flags (msg, SOUP_MESSAGE_NO_REDIRECT);
+	body = soup_test_session_send (session, msg, NULL, &error);
+	if (expected_status == SOUP_STATUS_NONE)
+		g_assert_error (error, G_TLS_ERROR, G_TLS_ERROR_BAD_CERTIFICATE);
+	else
+		g_assert_no_error (error);
 	soup_test_assert_message_status (msg, expected_status);
+	g_assert (enforced == expected_enforced);
+	g_clear_error (&error);
+	g_bytes_unref (body);
 	g_object_unref (msg);
 }
 
@@ -121,12 +146,15 @@ session_get_uri (SoupSession *session, const char *uri, SoupStatus expected_stat
 static void
 rewrite_message_uri (SoupMessage *msg)
 {
-	if (soup_uri_get_scheme (soup_message_get_uri (msg)) == SOUP_URI_SCHEME_HTTP)
-		soup_uri_set_port (soup_message_get_uri (msg), soup_uri_get_port (http_uri));
-	else if (soup_uri_get_scheme (soup_message_get_uri (msg)) == SOUP_URI_SCHEME_HTTPS)
-		soup_uri_set_port (soup_message_get_uri (msg), soup_uri_get_port (https_uri));
+	GUri *new_uri;
+	if (soup_uri_is_http (soup_message_get_uri (msg)))
+		new_uri = soup_uri_copy (soup_message_get_uri (msg), SOUP_URI_PORT, g_uri_get_port (http_uri), SOUP_URI_NONE);
+	else if (soup_uri_is_https (soup_message_get_uri (msg)))
+		new_uri = soup_uri_copy (soup_message_get_uri (msg), SOUP_URI_PORT, g_uri_get_port (https_uri), SOUP_URI_NONE);
 	else
 		g_assert_not_reached();
+	soup_message_set_uri (msg, new_uri);
+	g_uri_unref (new_uri);
 }
 
 static void
@@ -149,18 +177,12 @@ on_request_queued (SoupSession *session,
 static SoupSession *
 hsts_session_new (SoupHSTSEnforcer *enforcer)
 {
-	SoupSession *session;
+	SoupSession *session = soup_test_session_new (NULL);
 
 	if (enforcer)
-		session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
-						 SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
-						 SOUP_SESSION_ADD_FEATURE, enforcer,
-						 NULL);
+                soup_session_add_feature (session, SOUP_SESSION_FEATURE (enforcer));
 	else
-		session = soup_test_session_new (SOUP_TYPE_SESSION_ASYNC,
-						 SOUP_SESSION_USE_THREAD_CONTEXT, TRUE,
-						 SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_HSTS_ENFORCER,
-						 NULL);
+                soup_session_add_feature_by_type (session, SOUP_TYPE_HSTS_ENFORCER);
 
 	g_signal_connect (session, "request-queued", G_CALLBACK (on_request_queued), NULL);
 
@@ -173,15 +195,15 @@ do_hsts_basic_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
 
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
-	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
+	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 
 	/* The HSTS headers in the url above doesn't include
 	   subdomains, so the request should ask for the unchanged
 	   HTTP address below, to which the server will respond with a
 	   moved permanently status. */
-	session_get_uri (session, "http://subdomain.localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://subdomain.localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -191,11 +213,11 @@ do_hsts_expire_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
 
-	session_get_uri (session, "https://localhost/two-seconds", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/two-seconds", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	/* Wait for the policy to expire. */
 	g_usleep (3 * G_USEC_PER_SEC);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -205,9 +227,9 @@ do_hsts_delete_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
 
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
-	session_get_uri (session, "https://localhost/delete", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
+	session_get_uri (session, "https://localhost/delete", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -216,12 +238,12 @@ static void
 do_hsts_replace_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
-	session_get_uri (session, "https://localhost/two-seconds", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
+	session_get_uri (session, "https://localhost/two-seconds", SOUP_STATUS_OK, FALSE);
 	/* Wait for the policy to expire. */
 	g_usleep (3 * G_USEC_PER_SEC);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -230,15 +252,15 @@ static void
 do_hsts_update_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/three-seconds", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/three-seconds", SOUP_STATUS_OK, FALSE);
 	g_usleep (2 * G_USEC_PER_SEC);
-	session_get_uri (session, "https://localhost/three-seconds", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/three-seconds", SOUP_STATUS_OK, FALSE);
 	g_usleep (2 * G_USEC_PER_SEC);
 
 	/* At this point, 4 seconds have elapsed since setting the 3 seconds HSTS
 	   rule for the first time, and it should have expired by now, but since it
 	   was updated, it should still be valid. */
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -246,10 +268,10 @@ static void
 do_hsts_set_and_delete_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
-	session_get_uri (session, "https://localhost/delete", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
+	session_get_uri (session, "https://localhost/delete", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -258,10 +280,10 @@ static void
 do_hsts_no_hsts_header_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
-	session_get_uri (session, "https://localhost/no-sts-header", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
+	session_get_uri (session, "https://localhost/no-sts-header", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -270,12 +292,12 @@ static void
 do_hsts_persistency_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	soup_test_session_abort_unref (session);
 
 	session = hsts_session_new (NULL);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -283,11 +305,11 @@ static void
 do_hsts_subdomains_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/subdomains", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/subdomains", SOUP_STATUS_OK, FALSE);
 	/* The enforcer should cause the request to ask for an HTTPS
 	   uri, which will fail with an SSL error as there's no server
 	   in subdomain.localhost. */
-	session_get_uri (session, "http://subdomain.localhost", SOUP_STATUS_SSL_FAILED);
+	session_get_uri (session, "http://subdomain.localhost", SOUP_STATUS_NONE, TRUE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -299,7 +321,7 @@ do_hsts_superdomain_test (void)
 
 	SoupSession *session = hsts_session_new (enforcer);
 	/* This adds a long-lasting policy for localhost. */
-	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/long-lasting", SOUP_STATUS_OK, FALSE);
 
 	/* We want to set a policy with age = 0 for a subdomain, to test that the
 	   superdomain's policy is not removed. We cannot test this with a
@@ -310,7 +332,7 @@ do_hsts_superdomain_test (void)
 
 	/* This should work, as we have a long-lasting policy in place. If it fails,
 	   the subdomain policy has modified the superdomain's policy, which is wrong. */
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	g_object_unref (enforcer);
 }
 
@@ -318,9 +340,9 @@ static void
 do_hsts_multiple_headers_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/multiple-headers", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/multiple-headers", SOUP_STATUS_OK, FALSE);
 	g_usleep(2 * G_USEC_PER_SEC);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -328,8 +350,8 @@ static void
 do_hsts_insecure_sts_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "http://localhost/insecure", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://localhost/insecure", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -337,8 +359,8 @@ static void
 do_hsts_missing_values_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/missing-values", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "https://localhost/missing-values", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -346,8 +368,8 @@ static void
 do_hsts_invalid_values_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/invalid-values", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "https://localhost/invalid-values", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -358,8 +380,8 @@ do_hsts_extra_values_test (void)
 	for (i = 0; i < 2; i++) {
 		SoupSession *session = hsts_session_new (NULL);
 		char *uri = g_strdup_printf ("https://localhost/extra-values-%i", i);
-		session_get_uri (session, uri, SOUP_STATUS_OK);
-		session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+		session_get_uri (session, uri, SOUP_STATUS_OK, FALSE);
+		session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 		soup_test_session_abort_unref (session);
 		g_free (uri);
 	}
@@ -369,8 +391,8 @@ static void
 do_hsts_duplicated_directives_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/duplicated-directives", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "https://localhost/duplicated-directives", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -378,8 +400,8 @@ static void
 do_hsts_case_insensitive_header_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/case-insensitive-header", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/case-insensitive-header", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -387,8 +409,8 @@ static void
 do_hsts_case_insensitive_directives_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/case-insensitive-directives", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/case-insensitive-directives", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -397,8 +419,8 @@ do_hsts_optional_quotations_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
 
-	session_get_uri (session, "https://localhost/optional-quotations", SOUP_STATUS_OK);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/optional-quotations", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 
 	soup_test_session_abort_unref (session);
 }
@@ -407,8 +429,8 @@ static void
 do_hsts_ip_address_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://127.0.0.1/basic", SOUP_STATUS_OK);
-	session_get_uri (session, "http://127.0.0.1/", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "https://127.0.0.1/basic", SOUP_STATUS_OK, FALSE);
+	session_get_uri (session, "http://127.0.0.1/", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -416,11 +438,11 @@ static void
 do_hsts_utf8_address_test (void)
 {
 	SoupSession *session = hsts_session_new (NULL);
-	session_get_uri (session, "https://localhost/subdomains", SOUP_STATUS_OK);
+	session_get_uri (session, "https://localhost/subdomains", SOUP_STATUS_OK, FALSE);
 	/* The enforcer should cause the request to ask for an HTTPS
 	   uri, which will fail with an SSL error as there's no server
 	   in 食狮.中国.localhost. */
-	session_get_uri (session, "http://食狮.中国.localhost", SOUP_STATUS_SSL_FAILED);
+	session_get_uri (session, "http://食狮.中国.localhost", SOUP_STATUS_NONE, TRUE);
 	soup_test_session_abort_unref (session);
 }
 
@@ -430,9 +452,9 @@ do_hsts_session_policy_test (void)
 	SoupHSTSEnforcer *enforcer = soup_hsts_enforcer_new ();
 	SoupSession *session = hsts_session_new (enforcer);
 
-	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_MOVED_PERMANENTLY, FALSE);
 	soup_hsts_enforcer_set_session_policy (enforcer, "localhost", FALSE);
-	session_get_uri (session, "http://localhost", SOUP_STATUS_OK);
+	session_get_uri (session, "http://localhost", SOUP_STATUS_OK, TRUE);
 
 	soup_test_session_abort_unref (session);
 	g_object_unref (enforcer);
@@ -444,9 +466,9 @@ on_idna_test_enforcer_changed (SoupHSTSEnforcer *enforcer, SoupHSTSPolicy *old,
 	/* If NULL, then instead of replacing we're adding a new
 	 * policy and somewhere we're failing to canonicalize a hostname. */
 	g_assert_nonnull (old);
-	g_assert_cmpstr (old->domain, ==, new->domain);
+	g_assert_cmpstr (soup_hsts_policy_get_domain (old), ==, soup_hsts_policy_get_domain (new));
 	/*  Domains should not have punycoded segments at this point. */
-	g_assert_false (g_hostname_is_ascii_encoded (old->domain));
+	g_assert_false (g_hostname_is_ascii_encoded (soup_hsts_policy_get_domain (old)));
 }
 
 static void
@@ -538,7 +560,7 @@ do_hsts_get_policies_test (void)
 	g_assert_nonnull (policies);
 	g_assert_cmpint (g_list_length (policies), ==, 1);
 	policy = (SoupHSTSPolicy*)policies->data;
-	g_assert_cmpstr (policy->domain, ==, "gnome.org");
+	g_assert_cmpstr (soup_hsts_policy_get_domain (policy), ==, "gnome.org");
 	g_list_free_full (policies, (GDestroyNotify)soup_hsts_policy_free);
 
 	policy = soup_hsts_policy_new ("gnome.org", SOUP_HSTS_POLICY_MAX_AGE_PAST, FALSE);
@@ -548,7 +570,7 @@ do_hsts_get_policies_test (void)
 	policies = soup_hsts_enforcer_get_policies (enforcer, TRUE);
 	g_assert_cmpint (g_list_length (policies), ==, 1);
 	policy = (SoupHSTSPolicy*)policies->data;
-	g_assert_cmpstr (policy->domain, ==, "freedesktop.org");
+	g_assert_cmpstr (soup_hsts_policy_get_domain (policy), ==, "freedesktop.org");
 	g_list_free_full (policies, (GDestroyNotify)soup_hsts_policy_free);
 	g_object_unref(enforcer);
 }
@@ -603,11 +625,11 @@ main (int argc, char **argv)
 
 	ret = g_test_run ();
 
-	soup_uri_free (http_uri);
+	g_uri_unref (http_uri);
 	soup_test_server_quit_unref (server);
 
 	if (tls_available) {
-		soup_uri_free (https_uri);
+		g_uri_unref (https_uri);
 		soup_test_server_quit_unref (https_server);
 	}
 
