@@ -1,6 +1,7 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
 #include "test-utils.h"
+#include "soup-misc.h"
 
 #include <glib/gprintf.h>
 
@@ -12,7 +13,7 @@ static gboolean apache_running;
 #endif
 
 static SoupLogger *logger;
-static SoupBuffer *index_buffer;
+static GBytes *index_buffer;
 
 int debug_level;
 gboolean expect_warning, tls_available;
@@ -111,7 +112,7 @@ test_cleanup (void)
 	if (logger)
 		g_object_unref (logger);
 	if (index_buffer)
-		soup_buffer_free (index_buffer);
+		g_bytes_unref (index_buffer);
 
 	g_main_context_unref (g_main_context_default ());
 
@@ -122,15 +123,35 @@ void
 debug_printf (int level, const char *format, ...)
 {
 	va_list args;
+        static char last_char = '\n';
 
 	if (debug_level < level)
 		return;
 
+        /* Very hacky solution to outputing in the TAP format,
+         * every line starts with # */
+        if (last_char == '\n')
+                g_printf ("# ");
+        last_char = format[strlen (format) - 1];
+
 	va_start (args, format);
 	g_vprintf (format, args);
 	va_end (args);
 }
 
+gboolean
+have_curl(void) {
+	char *found;
+
+	found = g_find_program_in_path ("curl");
+	if (found) {
+		g_free (found);
+		return TRUE;
+	} else {
+		return FALSE;
+	}
+}
+
 #ifdef HAVE_APACHE
 
 static gboolean
@@ -145,6 +166,13 @@ apache_cmd (const char *cmd)
 	gboolean ok;
 
 	server_root = g_test_build_filename (G_TEST_BUILT, "", NULL);
+	if (!g_path_is_absolute (server_root)) {
+		char *abs_server_root;
+
+		abs_server_root = g_canonicalize_filename (server_root, NULL);
+		g_free (server_root);
+		server_root = abs_server_root;
+	}
 
 	cwd = g_get_current_dir ();
 #ifdef HAVE_APACHE_2_4
@@ -189,7 +217,9 @@ apache_cmd (const char *cmd)
 void
 apache_init (void)
 {
-	if (g_getenv ("SOUP_TESTS_IN_MAKE_CHECK"))
+	/* Set this environment variable if you are already running a
+	 * suitably-configured Apache server */
+	if (g_getenv ("SOUP_TESTS_ALREADY_RUNNING_APACHE"))
 		return;
 
 	if (!apache_cmd ("start")) {
@@ -224,18 +254,16 @@ apache_cleanup (void)
 #endif /* HAVE_APACHE */
 
 SoupSession *
-soup_test_session_new (GType type, ...)
+soup_test_session_new (const char *propname, ...)
 {
 	va_list args;
-	const char *propname;
 	SoupSession *session;
 	GTlsDatabase *tlsdb;
 	char *cafile;
 	GError *error = NULL;
 
-	va_start (args, type);
-	propname = va_arg (args, const char *);
-	session = (SoupSession *)g_object_new_valist (type, propname, args);
+	va_start (args, propname);
+	session = (SoupSession *)g_object_new_valist (SOUP_TYPE_SESSION, propname, args);
 	va_end (args);
 
 	if (tls_available) {
@@ -253,16 +281,14 @@ soup_test_session_new (GType type, ...)
 				g_assert_no_error (error);
 		}
 
-		g_object_set (G_OBJECT (session),
-			      SOUP_SESSION_TLS_DATABASE, tlsdb,
-			      NULL);
+		soup_session_set_tls_database (session, tlsdb);
 		g_clear_object (&tlsdb);
 	}
 
 	if (http_debug_level && !logger) {
-		SoupLoggerLogLevel level = MIN ((SoupLoggerLogLevel)http_debug_level, SOUP_LOGGER_LOG_BODY);
+		SoupLoggerLogLevel level = MIN ((SoupLoggerLogLevel)http_debug_level, SOUP_LOGGER_LOG_HEADERS);
 
-		logger = soup_logger_new (level, -1);
+		logger = soup_logger_new (level);
 	}
 
 	if (logger)
@@ -280,12 +306,96 @@ soup_test_session_abort_unref (SoupSession *session)
 	g_object_unref (session);
 }
 
+typedef struct {
+	GBytes *body;
+	GError *error;
+	gboolean done;
+} SendAsyncData;
+
+static void
+send_async_ready_cb (SoupSession   *session,
+		     GAsyncResult  *result,
+		     SendAsyncData *data)
+{
+	GInputStream *istream;
+	GOutputStream *ostream;
+
+	data->done = TRUE;
+	istream = soup_session_send_finish (session, result, &data->error);
+	if (!istream)
+		return;
+
+	ostream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
+	g_output_stream_splice (ostream,
+				istream,
+				G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
+				G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+				NULL,
+				&data->error);
+	data->body = g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (ostream));
+	g_object_unref (ostream);
+	g_object_unref (istream);
+}
+
+static void
+on_message_finished (SoupMessage *msg,
+                    gboolean    *message_finished)
+{
+        *message_finished = TRUE;
+}
+
+GBytes *
+soup_test_session_async_send (SoupSession  *session,
+			      SoupMessage  *msg,
+			      GCancellable *cancellable,
+			      GError      **error)
+{
+	gboolean message_finished = FALSE;
+	GMainContext *async_context = g_main_context_ref_thread_default ();
+	gulong signal_id;
+	SendAsyncData data = { NULL, NULL, FALSE };
+
+	signal_id = g_signal_connect (msg, "finished",
+                                     G_CALLBACK (on_message_finished), &message_finished);
+
+	soup_session_send_async (session, msg, G_PRIORITY_DEFAULT, cancellable,
+				 (GAsyncReadyCallback)send_async_ready_cb, &data);
+
+	while (!data.done || !message_finished)
+		g_main_context_iteration (async_context, TRUE);
+
+	g_signal_handler_disconnect (msg, signal_id);
+
+	if (data.error)
+		g_propagate_error (error, data.error);
+
+        g_main_context_unref (async_context);
+	return data.body;
+}
+
+guint
+soup_test_session_send_message (SoupSession *session,
+				SoupMessage *msg)
+{
+	GInputStream *stream;
+
+	stream = soup_session_send (session, msg, NULL, NULL);
+	if (stream)
+		g_object_unref (stream);
+
+	return soup_message_get_status (msg);
+}
+
 static void
 server_listen (SoupServer *server)
 {
 	GError *error = NULL;
+        SoupServerListenOptions options = 0;
 
-	soup_server_listen_local (server, 0, 0, &error);
+        if (g_getenv ("SOUP_TEST_NO_IPV6"))
+                options = SOUP_SERVER_LISTEN_IPV4_ONLY;
+
+	soup_server_listen_local (server, 0, options, &error);
 	if (error) {
 		g_printerr ("Unable to create server: %s\n", error->message);
 		exit (1);
@@ -327,6 +437,20 @@ run_server_thread (gpointer user_data)
 	return NULL;
 }
 
+void
+soup_test_server_run_in_thread (SoupServer *server)
+{
+	GThread *thread;
+
+	g_mutex_lock (&server_start_mutex);
+
+	thread = g_thread_new ("server_thread", run_server_thread, server);
+	g_cond_wait (&server_start_cond, &server_start_mutex);
+	g_mutex_unlock (&server_start_mutex);
+
+	g_object_set_data (G_OBJECT (server), "thread", thread);
+}
+
 SoupServer *
 soup_test_server_new (SoupTestServerOptions options)
 {
@@ -350,60 +474,53 @@ soup_test_server_new (SoupTestServerOptions options)
 		}
 	}
 
-	server = soup_server_new (SOUP_SERVER_TLS_CERTIFICATE, cert,
+	server = soup_server_new ("tls-certificate", cert,
 				  NULL);
 	g_clear_object (&cert);
 
 	g_object_set_data (G_OBJECT (server), "options", GUINT_TO_POINTER (options));
 
-	if (options & SOUP_TEST_SERVER_IN_THREAD) {
-		GThread *thread;
-
-		g_mutex_lock (&server_start_mutex);
-
-		thread = g_thread_new ("server_thread", run_server_thread, server);
-		g_cond_wait (&server_start_cond, &server_start_mutex);
-		g_mutex_unlock (&server_start_mutex);
-
-		g_object_set_data (G_OBJECT (server), "thread", thread);
-	} else if (!(options & SOUP_TEST_SERVER_NO_DEFAULT_LISTENER))
+	if (options & SOUP_TEST_SERVER_IN_THREAD)
+		soup_test_server_run_in_thread (server);
+	else if (!(options & SOUP_TEST_SERVER_NO_DEFAULT_LISTENER))
 		server_listen (server);
 
 	return server;
 }
 
-static SoupURI *
+static GUri *
 find_server_uri (SoupServer *server, const char *scheme, const char *host)
 {
 	GSList *uris, *u;
-	SoupURI *uri, *ret_uri = NULL;
+	GUri *uri, *ret_uri = NULL;
 
 	uris = soup_server_get_uris (server);
 	for (u = uris; u; u = u->next) {
 		uri = u->data;
 
-		if (scheme && strcmp (uri->scheme, scheme) != 0)
+		if (scheme && strcmp (g_uri_get_scheme (uri), scheme) != 0)
 			continue;
-		if (host && strcmp (uri->host, host) != 0)
+		if (host && strcmp (g_uri_get_host (uri), host) != 0)
 			continue;
 
-		ret_uri = soup_uri_copy (uri);
+		ret_uri = g_uri_ref (uri);
 		break;
 	}
-	g_slist_free_full (uris, (GDestroyNotify)soup_uri_free);
+	g_slist_free_full (uris, (GDestroyNotify)g_uri_unref);
 
 	return ret_uri;
 }
 
-static SoupURI *
+static GUri *
 add_listener (SoupServer *server, const char *scheme, const char *host)
 {
 	SoupServerListenOptions options = 0;
 	GError *error = NULL;
 
-	if (!g_strcmp0 (scheme, SOUP_URI_SCHEME_HTTPS))
+	if (!g_strcmp0 (scheme, "https"))
 		options |= SOUP_SERVER_LISTEN_HTTPS;
-	if (!g_strcmp0 (host, "127.0.0.1"))
+
+	if (!g_strcmp0 (host, "127.0.0.1") || g_getenv ("SOUP_TEST_NO_IPV6"))
 		options |= SOUP_SERVER_LISTEN_IPV4_ONLY;
 	else if (!g_strcmp0 (host, "::1"))
 		options |= SOUP_SERVER_LISTEN_IPV6_ONLY;
@@ -422,7 +539,7 @@ typedef struct {
 	const char *scheme;
 	const char *host;
 
-	SoupURI *uri;
+	GUri *uri;
 } AddListenerData;
 
 static gboolean
@@ -438,12 +555,12 @@ add_listener_in_thread (gpointer user_data)
 	return FALSE;
 }
 
-SoupURI *
+GUri *
 soup_test_server_get_uri (SoupServer    *server,
 			  const char    *scheme,
 			  const char    *host)
 {
-	SoupURI *uri;
+	GUri *uri;
 	GMainLoop *loop;
 
 	uri = find_server_uri (server, scheme, host);
@@ -560,102 +677,43 @@ async_as_sync_callback (GObject      *object,
 	g_main_loop_quit (data->loop);
 }
 
-typedef struct {
-	SoupRequest  *req;
-	GCancellable *cancellable;
-	SoupTestRequestFlags flags;
-} CancelData;
-
-static CancelData *
-create_cancel_data (SoupRequest          *req,
-		    GCancellable         *cancellable,
-		    SoupTestRequestFlags  flags)
-{
-	CancelData *cancel_data;
-
-	if (!flags)
-		return NULL;
-
-	cancel_data = g_slice_new0 (CancelData);
-	cancel_data->flags = flags;
-	if (flags & SOUP_TEST_REQUEST_CANCEL_MESSAGE && SOUP_IS_REQUEST_HTTP (req))
-		cancel_data->req = g_object_ref (req);
-	else if (flags & SOUP_TEST_REQUEST_CANCEL_CANCELLABLE)
-		cancel_data->cancellable = g_object_ref (cancellable);
-	return cancel_data;
-}
-
-inline static void
-cancel_message_or_cancellable (CancelData *cancel_data)
-{
-	if (cancel_data->flags & SOUP_TEST_REQUEST_CANCEL_MESSAGE) {
-		SoupRequest *req = cancel_data->req;
-		SoupMessage *msg = soup_request_http_get_message (SOUP_REQUEST_HTTP (req));
-		soup_session_cancel_message (soup_request_get_session (req), msg,
-					     SOUP_STATUS_CANCELLED);
-		g_object_unref (msg);
-		g_object_unref (req);
-	} else if (cancel_data->flags & SOUP_TEST_REQUEST_CANCEL_CANCELLABLE) {
-		g_cancellable_cancel (cancel_data->cancellable);
-		g_object_unref (cancel_data->cancellable);
-	}
-	g_slice_free (CancelData, cancel_data);
-}
-
 static gboolean
-cancel_request_timeout (gpointer data)
+cancel_request_timeout (GCancellable *cancellable)
 {
-	cancel_message_or_cancellable ((CancelData *) data);
+	g_cancellable_cancel (cancellable);
 	return FALSE;
 }
 
-static gpointer
-cancel_request_thread (gpointer data)
-{
-	g_usleep (100000); /* .1s */
-	cancel_message_or_cancellable ((CancelData *) data);
-	return NULL;
-}
-
 GInputStream *
-soup_test_request_send (SoupRequest   *req,
+soup_test_request_send (SoupSession   *session,
+			SoupMessage   *msg,
 			GCancellable  *cancellable,
 			guint          flags,
 			GError       **error)
 {
 	AsyncAsSyncData data;
 	GInputStream *stream;
-	CancelData *cancel_data = create_cancel_data (req, cancellable, flags);
-
-	if (SOUP_IS_SESSION_SYNC (soup_request_get_session (req))) {
-		GThread *thread;
-
-		if (cancel_data)
-			thread = g_thread_new ("cancel_request_thread", cancel_request_thread,
-					       cancel_data);
-		stream = soup_request_send (req, cancellable, error);
-		if (cancel_data)
-			g_thread_unref (thread);
-		return stream;
-	}
 
 	data.loop = g_main_loop_new (g_main_context_get_thread_default (), FALSE);
-	if (cancel_data &&
-	    (flags & SOUP_TEST_REQUEST_CANCEL_SOON || flags & SOUP_TEST_REQUEST_CANCEL_IMMEDIATE)) {
+	if (flags & SOUP_TEST_REQUEST_CANCEL_SOON || flags & SOUP_TEST_REQUEST_CANCEL_IMMEDIATE) {
 		guint interval = flags & SOUP_TEST_REQUEST_CANCEL_SOON ? 100 : 0;
-		g_timeout_add_full (G_PRIORITY_HIGH, interval, cancel_request_timeout, cancel_data, NULL);
+		g_timeout_add_full (G_PRIORITY_HIGH, interval,
+				    (GSourceFunc)cancel_request_timeout,
+				    g_object_ref (cancellable), g_object_unref);
 	}
-	if (cancel_data && (flags & SOUP_TEST_REQUEST_CANCEL_PREEMPTIVE))
-		cancel_message_or_cancellable (cancel_data);
-	soup_request_send_async (req, cancellable, async_as_sync_callback, &data);
+	if (flags & SOUP_TEST_REQUEST_CANCEL_PREEMPTIVE)
+		g_cancellable_cancel (cancellable);
+
+	soup_session_send_async (session, msg, G_PRIORITY_DEFAULT, cancellable,
+				 async_as_sync_callback, &data);
 	g_main_loop_run (data.loop);
 
-	stream = soup_request_send_finish (req, data.result, error);
+	stream = soup_session_send_finish (session, data.result, error);
 
-	if (cancel_data && (flags &  SOUP_TEST_REQUEST_CANCEL_AFTER_SEND_FINISH)) {
+	if (flags & SOUP_TEST_REQUEST_CANCEL_AFTER_SEND_FINISH) {
 		GMainContext *context;
 
-		cancel_message_or_cancellable (cancel_data);
+		g_cancellable_cancel (cancellable);
 
 		context = g_main_loop_get_context (data.loop);
 		while (g_main_context_pending (context))
@@ -669,8 +727,7 @@ soup_test_request_send (SoupRequest   *req,
 }
 
 gboolean
-soup_test_request_read_all (SoupRequest   *req,
-			    GInputStream  *stream,
+soup_test_request_read_all (GInputStream  *stream,
 			    GCancellable  *cancellable,
 			    GError       **error)
 {
@@ -678,43 +735,30 @@ soup_test_request_read_all (SoupRequest   *req,
 	AsyncAsSyncData data;
 	gsize nread;
 
-	if (!SOUP_IS_SESSION_SYNC (soup_request_get_session (req)))
-		data.loop = g_main_loop_new (g_main_context_get_thread_default (), FALSE);
-	else
-		data.loop = NULL;
+        data.loop = g_main_loop_new (g_main_context_get_thread_default (), FALSE);
 
 	do {
-		if (!data.loop) {
-			nread = g_input_stream_read (stream, buf, sizeof (buf),
-						     cancellable, error);
-		} else {
-			g_input_stream_read_async (stream, buf, sizeof (buf),
-						   G_PRIORITY_DEFAULT, cancellable,
-						   async_as_sync_callback, &data);
-			g_main_loop_run (data.loop);
-			nread = g_input_stream_read_finish (stream, data.result, error);
-			g_object_unref (data.result);
-		}
+                g_input_stream_read_async (stream, buf, sizeof (buf),
+					   G_PRIORITY_DEFAULT, cancellable,
+					   async_as_sync_callback, &data);
+                g_main_loop_run (data.loop);
+                nread = g_input_stream_read_finish (stream, data.result, error);
+                g_object_unref (data.result);
 	} while (nread > 0);
 
-	if (data.loop)
-		g_main_loop_unref (data.loop);
+	g_main_loop_unref (data.loop);
 
 	return nread == 0;
 }
 
 gboolean
-soup_test_request_close_stream (SoupRequest   *req,
-				GInputStream  *stream,
+soup_test_request_close_stream (GInputStream  *stream,
 				GCancellable  *cancellable,
 				GError       **error)
 {
 	AsyncAsSyncData data;
 	gboolean ok;
 
-	if (SOUP_IS_SESSION_SYNC (soup_request_get_session (req)))
-		return g_input_stream_close (stream, cancellable, error);
-
 	data.loop = g_main_loop_new (g_main_context_get_thread_default (), FALSE);
 
 	g_input_stream_close_async (stream, G_PRIORITY_DEFAULT, cancellable,
@@ -729,6 +773,33 @@ soup_test_request_close_stream (SoupRequest   *req,
 	return ok;
 }
 
+GBytes *
+soup_test_session_send (SoupSession   *session,
+			SoupMessage   *msg,
+			GCancellable  *cancellable,
+			GError       **error)
+{
+	GInputStream *istream;
+	GOutputStream *ostream;
+	GBytes *body;
+
+	istream = soup_session_send (session, msg, cancellable, error);
+	if (!istream)
+		return NULL;
+
+	ostream = g_memory_output_stream_new (NULL, 0, g_realloc, g_free);
+	g_output_stream_splice (ostream,
+				istream,
+				G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
+				G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+				NULL, NULL);
+	body = g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (ostream));
+	g_object_unref (ostream);
+	g_object_unref (istream);
+
+	return body;
+}
+
 void
 soup_test_register_resources (void)
 {
@@ -755,7 +826,7 @@ soup_test_register_resources (void)
 	registered = TRUE;
 }
 
-SoupBuffer *
+GBytes *
 soup_test_load_resource (const char  *name,
 			 GError     **error)
 {
@@ -768,16 +839,10 @@ soup_test_load_resource (const char  *name,
 	bytes = g_resources_lookup_data (path, G_RESOURCE_LOOKUP_FLAGS_NONE, error);
 	g_free (path);
 
-	if (!bytes)
-		return NULL;
-
-	return soup_buffer_new_with_owner (g_bytes_get_data (bytes, NULL),
-					   g_bytes_get_size (bytes),
-					   bytes,
-					   (GDestroyNotify) g_bytes_unref);
+        return bytes;
 }
 
-SoupBuffer *
+GBytes *
 soup_test_get_index (void)
 {
 	if (!index_buffer) {
@@ -793,7 +858,7 @@ soup_test_get_index (void)
 		}
 		g_free (path);
 
-		index_buffer = soup_buffer_new (SOUP_MEMORY_TAKE, contents, length);
+		index_buffer = g_bytes_new_take (contents, length);
 	}
 
 	return index_buffer;
